import numpy as np

# physical constants
L = 2.5e6  # latent heat of condensation [J/kg]
g = 9.80665  # gravitational acceleration [m/s^2]
kappa = 2.0/7.0
rdgas = 287.04 # gas constant dry air [J/kg/K]
rvgas = 461.50 # gas constant water vapor [J/kg/K]
cp = 1005.7  # specific heat capacity dry air [J/kg/K]
p_ref = 1e5 # reference pressure [Pa]

# scripts to calculate atmospheric variables

def vertical_integral(data, dlev):
# vertical integral with respect to sigma
    data = -np.sum(data * dlev[:, None].T, axis=1)
    return data

def calc_enthalpy(T, q, dlev):
# column-averaged enthalpy expressed in K (or K/s if inputs are tendencies)
    k = (T + (L/cp) * q)
    k = vertical_integral(k, dlev)/vertical_integral(1,dlev)
    return k

def calc_energy_conservation(T, q, dlev, ps):
# rate of gain of energy over column in W/m^2
# primary inputs are tendencies of T and q
    k = cp*T + L*q
    k = ps/g*vertical_integral(cp*T+L*q, dlev)
    return k

def calc_precip(q, dlev, ps):
# surface precipitation rate given tendency of specific humidity
    precip = ps/g*vertical_integral(q, dlev)  
    return precip

def calc_theta(T, lev, ps):
# potential temperature given temperature, sigma, and surface pressure
# used for a single profile (so ps is a scalar)
    p = lev*ps
    theta = T * np.power(p_ref/p, kappa)
    return theta

def simple_sat_vap(T):
# simple formula for saturation vapor pressure given temperature
    e0 = 610.78 # Pa
    T0 = 273.16 # K
    esat = e0*np.exp(-L/rvgas*(1.0/T-1.0/T0))
    return esat

def vap(p, q):
# vapor pressure given pressure and specific humidity
    eps = rdgas/rvgas;
    e = p*q/(eps+(1.0-eps)*q)
    return e

