import numpy as np
import matplotlib
import matplotlib.gridspec as gridspec
matplotlib.use('Agg')  # so figs just print to file. Needs to come before pyplot
import matplotlib.pyplot as plt
import scipy.stats
from sklearn import metrics
import pickle
import os
import sys
import src.ml_load as ml_load
import src.atmos_physics as atmos_physics
import warnings

warnings.filterwarnings("ignore", category=DeprecationWarning) 

# unpack routines
unpack_x = ml_load.unpack_x
unpack_y = ml_load.unpack_y
pack_y = ml_load.pack_y

# plotting defaults
matplotlib.rcParams['agg.path.chunksize'] = 10000
matplotlib.rcParams.update({'font.size': 8})

# convert tendencies from K/s, kg/kg/s and kg/m^2/s to K/day, g/kg/day and mm/day for plots with unit labels
# note for precipitation kg/m^2 goes to mm
per_day = 3600*24
kg_to_g = 1000

# number of points for precipitation scatter plots
num_scatter = 10000 

# ---   META PLOTTING SCRIPTS  --- #


def PlotAllFigs(est_str, datafile, do_nn, figpath, n_trn_exs=None, 
                rain_only=False, no_cos=False, no_ps=False, use_rh=False,
                conv_cond=False):
    # Open the estimator and the preprocessing scheme
    est_eval, _, errors, x_ppi, y_ppi, x_pp, y_pp, lat, lev, dlev = \
        pickle.load(open('/net/esker/archive1/pog/mldata/regressors/' + est_str + '.pkl', 'rb'))

 

    # Load the data from the training/testing file
    x_scl, ypred_scl, ytrue_scl, x, ypred, ytrue, Pout = \
        ml_load.get_x_y_pred_true(est_str, datafile, min_lev=min(lev),
                                 n_trn_exs=n_trn_exs, rain_only=rain_only, 
                                 no_cos=no_cos, no_ps=no_ps, use_rh=use_rh)

    print('Size of global dataset for plots after subsampling: ', x_scl.shape[0])

    # also load rain_only case
    indeq = int(lat.shape[0]/2)
    x_scl_rain, ypred_scl_rain, ytrue_scl_rain, x_rain, ypred_rain, ytrue_rain, Pout_rain = \
        ml_load.get_x_y_pred_true(est_str, datafile, min_lev=min(lev),
                                 n_trn_exs=n_trn_exs, rain_only=True,
                                 no_cos=no_cos, no_ps=no_ps, use_rh=use_rh)

    # also load equatorial, rain_only case
    x_scl_eqrain, ypred_scl_eqrain, ytrue_scl_eqrain, x_eqrain, ypred_eqrain, ytrue_eqrain, Pout_eqrain = \
        ml_load.get_x_y_pred_true(est_str, datafile, min_lev=min(lev),
                                 all_lats=False, ind_lat=indeq, 
                                 n_trn_exs=n_trn_exs, rain_only=True,
                                 no_cos=no_cos, no_ps=no_ps, use_rh=use_rh)


    # Do plotting
    print('Beginning to make plots and writing log files')

    # Create directory if it does not exist
    if not os.path.exists(figpath):
        os.makedirs(figpath)


    # Write test/train score to file for future reference
    text_file = open(figpath+'scores.txt', 'w')
    text_file.write('Score: %f' % est_eval.score(x_scl, ytrue_scl))
    text_file.close()

    # Plot the enthalpy conservation
    plot_enthalpy(x, ytrue, ypred, dlev, figpath)

    print('Beginning to make contour plots...')
    make_contour_plots(figpath, x_ppi, y_ppi, x_pp, y_pp, est_eval, lat, lev,
                       datafile, n_trn_exs, rain_only, no_cos, no_ps, use_rh)

    # test Pout accurate
    plot_precip_check(figpath, x, ytrue, Pout, dlev)

    print('Beginning to make importance plots...')
    if not no_ps: # these routines involve predicts that would have to be altered for the no_ps case
       # plot importance of features

       plot_importance_permute(figpath, x_scl, ytrue_scl, est_eval, lev, dlev, use_rh)
       plot_importance_precip_permute(figpath, x_scl, ytrue_scl, est_eval, lev, dlev, x_ppi, x_pp, y_ppi, y_pp, use_rh)

       plot_importance_permute(figpath+'rain_only_', x_scl_rain, ytrue_scl_rain, est_eval, lev, dlev, use_rh)
       plot_importance_precip_permute(figpath+'rain_only_', x_scl_rain, ytrue_scl_rain, est_eval, lev, dlev, x_ppi, x_pp, y_ppi, y_pp, use_rh)

       plot_importance_permute(figpath+'equator_rain_only_', x_scl_eqrain, ytrue_scl_eqrain, est_eval, lev, dlev, use_rh)
       plot_importance_precip_permute(figpath+'equator_rain_only_', x_scl_eqrain, ytrue_scl_eqrain, est_eval, lev, dlev, x_ppi, x_pp, y_ppi, y_pp, use_rh)

       if not do_nn:
        plot_importance(figpath, est_eval, lev, dlev, use_rh)
       # plot linear response 
        plot_linear_response(figpath, x, est_eval, lev, dlev, x_ppi, x_pp, y_ppi, y_pp, use_rh)
        plot_linear_response(figpath+'rain_only_', x_rain, est_eval, lev, dlev, x_ppi, x_pp, y_ppi, y_pp, use_rh)
        plot_linear_response(figpath+'equator_rain_only_', x_eqrain, est_eval, lev, dlev, x_ppi, x_pp, y_ppi, y_pp, use_rh)

    # Plot model errors over iteration history for neural net
    if do_nn:
     plot_model_error_over_time(errors, est_str, figpath)

    # Plot historgram showing how scaling changed character of input and output
    # data
    check_scaling_distribution(x, x_scl, ytrue, ytrue_scl, lat,
                               lev, figpath, use_rh=use_rh)
    # Plot histogram showing how well true and predicted values match
    check_output_distribution(ytrue, ytrue_scl, ypred, ypred_scl,
                              lat, lev, figpath)
    # Plot means and standard deviations
    plot_means_stds(ytrue, ypred, lev, figpath)
    # Plot correlation coefficient, explained variance, and rmse
    plot_error_stats(ytrue, ypred, lev, figpath)
    # Plot a "time series" of precipitaiton
    plot_precip(x, ytrue, ypred, dlev, figpath)
    # Plot a scatter plot of true vs predicted precip 
    plot_scatter(x[:num_scatter], ytrue[:num_scatter], ypred[:num_scatter], lev, dlev, conv_cond, figpath)
    # Plot some example profiles
    plot_sample_profiles(20, x, ytrue, ypred, lev, figpath)
    print('Done!')

def plot_precip_check(figpath, x, ytrue, Pout, dlev):
    # plot precipitation from GCM versus precipitation calculated from tendencies
    precip_true = atmos_physics.calc_precip(unpack_y(ytrue, 'q'), 
                                            dlev,
                                            unpack_x(x, 'ps'))
    fig = plt.figure(figsize=(3.0,2.25))
    plt.scatter(Pout[:num_scatter]*per_day, precip_true[:num_scatter]*per_day, s=5, c='b', alpha=0.2)
    xmin = np.min(Pout[:num_scatter])*per_day
    xmax = np.max(Pout[:num_scatter])*per_day
    ymin = np.min(precip_true[:num_scatter])*per_day
    ymax = np.max(precip_true[:num_scatter])*per_day
    xymin = np.min([xmin, ymin])
    xymax = np.max([xmax, ymax])
    plt.xlim(xymin, xymax)
    plt.ylim(xymin, xymax)
    plt.plot([xymin, xymax], [xymin, xymax], color='k', ls='--')
    plt.xlabel('Precip from GCM (mm/day)')
    plt.ylabel('Precip from tendencies (mm/day)')
    ax = plt.gca()
    ax.spines['right'].set_color('none')
    ax.spines['top'].set_color('none')
    plt.tight_layout() # avoid overlap
    fig.savefig(figpath + 'precip_from_tendencies_verification.eps', bbox_inches='tight')
    plt.close()

def plot_importance(figpath, est_eval, lev, dlev, use_rh):

    # weighted averaged of decrease in variance
    importance = est_eval.feature_importances_

    # simple estimate of standard error of the mean of feature importance across trees
    estimators = est_eval.estimators_
    stderr = np.std([tree.feature_importances_ for tree in estimators], axis=0)/np.sqrt(len(estimators))

    # account for variations in interlevel spacing
    rescale = 1.0/dlev

    fig = plt.figure(figsize=(3.0,2.25))

    plt.plot(lev*0, lev, 'k:', lw=0.5)

    plt.errorbar(rescale*unpack_x(importance, 'T', axis=0), lev, 
                 xerr=rescale*unpack_x(stderr, 'T', axis=0), 
                 label='T')

    if use_rh:
     plt.errorbar(rescale*unpack_x(importance, 'q', axis=0), lev, 
              xerr=rescale*unpack_x(stderr, 'q', axis=0), 
              label='rh')
    else:
     plt.errorbar(rescale*unpack_x(importance, 'q', axis=0), lev, 
              xerr=rescale*unpack_x(stderr, 'q', axis=0), 
              label='q')


    plt.ylim([np.amax(lev), np.amin(lev)])
    plt.ylabel('$\sigma$')
    plt.xlabel('Feature importance')
    plt.legend(loc="upper right")
    plt.legend(frameon=False)
    ax = plt.gca()
    ax.spines['right'].set_color('none')
    ax.spines['top'].set_color('none')
    plt.tight_layout() # avoid overlap
    fig.savefig(figpath + 'feature_importance.eps', bbox_inches='tight')
    plt.close()

    text_file = open(figpath+'feature_importance.txt', 'w')
    text_file.write('importance for ps: %f \n' % unpack_x(importance, 'ps', axis=0))
    text_file.write('sum of importance for q: %f \n' % np.sum(unpack_x(importance, 'q', axis=0)))
    text_file.write('sum of importance for T: %f \n' % np.sum(unpack_x(importance, 'T', axis=0)))
    text_file.write('standard error of mean importance of ps across trees: %f \n' %  unpack_x(stderr, 'ps', axis=0))
    text_file.close()



def plot_importance_permute(figpath, x_scl, ytrue_scl, est_eval, lev, dlev, use_rh):

    acc = metrics.r2_score(ytrue_scl, est_eval.predict(x_scl))
    importance = np.zeros(x_scl.shape[1])
    for i in range(x_scl.shape[1]):
        x_scl_shuff = x_scl.copy()
        np.random.shuffle(x_scl_shuff[:, i])
        shuff_acc = metrics.r2_score(ytrue_scl, est_eval.predict(x_scl_shuff))
        importance[i] = (acc-shuff_acc)/acc

    # account for variations in interlevel spacing
    rescale = 1.0/dlev

    fig = plt.figure(figsize=(3.0,2.25))

    plt.plot(lev*0, lev, 'k:', lw=0.5)

    plt.plot(rescale*unpack_x(importance, 'T', axis=0), lev, label='T')

    if use_rh:
     plt.plot(rescale*unpack_x(importance, 'q', axis=0), lev, label='rh')
    else:
     plt.plot(rescale*unpack_x(importance, 'q', axis=0), lev, label='q')

    text_file = open(figpath+'feature_importance_permute.txt', 'w')
    text_file.write('importance for ps: %f \n' % unpack_x(importance, 'ps', axis=0))
    text_file.close()

    plt.ylim([np.amax(lev), np.amin(lev)])
    plt.ylabel('$\sigma$')
    plt.xlabel('Feature importance')
    plt.legend(loc="upper right")
    plt.legend(frameon=False)
    ax = plt.gca()
    ax.spines['right'].set_color('none')
    ax.spines['top'].set_color('none')
    plt.tight_layout() # avoid overlap
    fig.savefig(figpath + 'feature_importance_permute.eps', bbox_inches='tight')
    plt.close()

def plot_importance_precip_permute(figpath, x_scl, ytrue_scl, est_eval, lev, dlev, x_ppi, x_pp, y_ppi, y_pp, use_rh):

    x = ml_load.inverse_transform_data(x_ppi, x_pp, x_scl, lev)
    ytrue = ml_load.inverse_transform_data(y_ppi, y_pp, ytrue_scl, lev)
    precip_true = atmos_physics.calc_precip(unpack_y(ytrue, 'q'), 
                                            dlev,
                                            unpack_x(x, 'ps'))

    ypred_scl = est_eval.predict(x_scl)
    ypred = ml_load.inverse_transform_data(y_ppi, y_pp, ypred_scl, lev)
    precip_pred = atmos_physics.calc_precip(unpack_y(ypred, 'q'),
                                            dlev,
                                            unpack_x(x, 'ps'))

    acc = metrics.r2_score(precip_true, precip_pred)

    importance = np.zeros(x_scl.shape[1])
    for i in range(x_scl.shape[1]):
        x_scl_shuff = x_scl.copy()
        np.random.shuffle(x_scl_shuff[:, i])
        y_scl_shuff = est_eval.predict(x_scl_shuff)
        y_shuff = ml_load.inverse_transform_data(y_ppi, y_pp, y_scl_shuff, lev)
        # use unshuffled ps when calculating precip as only measuring effect 
        # of shuffling ps on predicting temperature and moisture tendencies
        precip_shuff = atmos_physics.calc_precip(unpack_y(y_shuff, 'q'), 
                                                 dlev,
                                                 unpack_x(x, 'ps'))
        shuff_acc = metrics.r2_score(precip_true, precip_shuff)
        importance[i] = (acc-shuff_acc)/acc

    # account for variations in interlevel spacing
    rescale = 1.0/dlev

    # use sqrt to compare with linear_response
    importance = np.sqrt(np.absolute(importance))

    fig = plt.figure(figsize=(3.0,2.25))

    plt.plot(lev*0, lev, 'k:', lw=0.5)

    plt.plot(rescale*unpack_x(importance, 'T', axis=0), lev, label='T')

    if use_rh:
     plt.plot(rescale*unpack_x(importance, 'q', axis=0), lev, label='rh')
    else:
     plt.plot(rescale*unpack_x(importance, 'q', axis=0), lev, label='q')

    text_file = open(figpath+'feature_importance_precip_permute.txt', 'w')
    text_file.write('importance for ps: %f \n' % unpack_x(importance, 'ps', axis=0))
    text_file.close()

    plt.ylim([np.amax(lev), np.amin(lev)])
    plt.ylabel('$\sigma$')
    plt.xlabel('Feature importance')
    plt.legend(loc="upper right")
    plt.legend(frameon=False)
    ax = plt.gca()
    ax.spines['right'].set_color('none')
    ax.spines['top'].set_color('none')
    plt.tight_layout() # avoid overlap
    fig.savefig(figpath + 'feature_importance_precip_permute.eps', bbox_inches='tight')
    plt.close()


def plot_linear_response(figpath, x, est_eval, lev, dlev, x_ppi, x_pp, y_ppi, y_pp, use_rh=False):

    # uses central difference of total width 1K, 1 g/kg

    sensitivity = np.zeros(x.shape[1])
    sensitivity_stderr = np.zeros(x.shape[1])
    sensitivity_check_linear = np.zeros(x.shape[1])
    sensitivity_stderr_check_linear = np.zeros(x.shape[1])
    perturb = np.zeros(x.shape[1])

    N = int((x.shape[1]-1)/2)
    perturb[0:N] = 1 # 1 K
    if use_rh:
     perturb[N:2*N] = 0.05 # 5% RH
    else:
     perturb[N:2*N] = 0.001 # 1 g/kg 
    perturb[2*N] = 50 # 0.5 hPa

    for i in range(x.shape[1]):
     x_perturb_pos = x.copy()
     x_perturb_neg = x.copy()
     x_perturb_pos[:,i] = x_perturb_pos[:,i]+0.5*perturb[i]
     x_perturb_neg[:,i] = x_perturb_neg[:,i]-0.5*perturb[i]
     x_perturb_pos_scl = ml_load.transform_data(x_ppi, x_pp, x_perturb_pos, lev)
     x_perturb_neg_scl = ml_load.transform_data(x_ppi, x_pp, x_perturb_neg, lev)
     y_perturb_pos_scl = est_eval.predict(x_perturb_pos_scl)
     y_perturb_neg_scl = est_eval.predict(x_perturb_neg_scl)
     y_perturb_pos = ml_load.inverse_transform_data(y_ppi, y_pp, y_perturb_pos_scl, lev)
     y_perturb_neg = ml_load.inverse_transform_data(y_ppi, y_pp, y_perturb_neg_scl, lev)
     precip_perturb_pos = atmos_physics.calc_precip(unpack_y(y_perturb_pos, 'q'), dlev, unpack_x(x_perturb_pos, 'ps'))
     precip_perturb_neg = atmos_physics.calc_precip(unpack_y(y_perturb_neg, 'q'), dlev, unpack_x(x_perturb_neg, 'ps'))
     sensitivity[i] = np.mean(precip_perturb_pos - precip_perturb_neg)
     sensitivity_stderr[i] = np.std(precip_perturb_pos - precip_perturb_neg)/np.sqrt(x.shape[0]) 
     # check linearity by using double the perturbation size and halving the resulting sensitivity
     x_perturb_pos = x.copy()
     x_perturb_neg = x.copy()
     x_perturb_pos[:,i] = x_perturb_pos[:,i]+1.0*perturb[i]
     x_perturb_neg[:,i] = x_perturb_neg[:,i]-1.0*perturb[i]
     x_perturb_pos_scl = ml_load.transform_data(x_ppi, x_pp, x_perturb_pos, lev)
     x_perturb_neg_scl = ml_load.transform_data(x_ppi, x_pp, x_perturb_neg, lev)
     y_perturb_pos_scl = est_eval.predict(x_perturb_pos_scl)
     y_perturb_neg_scl = est_eval.predict(x_perturb_neg_scl)
     y_perturb_pos = ml_load.inverse_transform_data(y_ppi, y_pp, y_perturb_pos_scl, lev)
     y_perturb_neg = ml_load.inverse_transform_data(y_ppi, y_pp, y_perturb_neg_scl, lev)
     precip_perturb_pos = atmos_physics.calc_precip(unpack_y(y_perturb_pos, 'q'), dlev, unpack_x(x_perturb_pos, 'ps'))
     precip_perturb_neg = atmos_physics.calc_precip(unpack_y(y_perturb_neg, 'q'), dlev, unpack_x(x_perturb_neg, 'ps'))
     sensitivity_check_linear[i] = 0.5*np.mean(precip_perturb_pos - precip_perturb_neg)
     sensitivity_stderr_check_linear[i] = 0.5*np.std(precip_perturb_pos - precip_perturb_neg)/np.sqrt(x.shape[0]) 

    # account for variations in model level spacing
    dlev_ref = 0.05 # roughly 50hPa
    rescale = dlev_ref/dlev

    # plot linear response on its own
    fig = plt.figure(figsize=(3.0,2.25))

    plt.plot(lev*0, lev, 'k:', lw=0.5)

    plt.plot(rescale*unpack_x(sensitivity, 'T', axis=0)*per_day, lev, 'C0', label='T:1K over d$\sigma$=0.05')
    plt.plot(rescale*unpack_x(sensitivity_check_linear, 'T', axis=0)*per_day, lev, 'C0:')

    if use_rh:
     plt.plot(rescale*unpack_x(sensitivity, 'q', axis=0)*per_day, lev, 'C1', label='rh:5% over d$\sigma$=0.05')
     plt.plot(rescale*unpack_x(sensitivity_check_linear, 'q', axis=0)*per_day, lev, 'C1:')
    else:
     plt.plot(rescale*unpack_x(sensitivity, 'q', axis=0)*per_day, lev, 'C1', label='q:1g/kg over d$\sigma$=0.05')
     plt.plot(rescale*unpack_x(sensitivity_check_linear, 'q', axis=0)*per_day, lev, 'C1:')

    text_file = open(figpath+'linear_response.txt', 'w')
    text_file.write('sensitivity in mm/day for 0.5hPa of ps: %f \n' % (unpack_x(sensitivity, 'ps', axis=0)*per_day))
    text_file.write('standard error of sensitivity in mm/day for 0.5hPa of ps: %f \n' % (unpack_x(sensitivity_stderr, 'ps', axis=0)*per_day))
    text_file.write('sensitivity when double perturbation and halve sensitivity: %f \n' % (unpack_x(sensitivity_check_linear, 'ps', axis=0)*per_day))
    text_file.close()


    plt.ylim([np.amax(lev), np.amin(lev)])
    plt.ylabel('Vertical level ($\sigma$)')
    plt.xlabel('Sensitivity in mm $\mathregular{day^{-1}}$ per perturbation')
    plt.legend(loc="upper right")
    plt.legend(frameon=False)
    ax = plt.gca()
    ax.spines['right'].set_color('none')
    ax.spines['top'].set_color('none')
    plt.tight_layout() # avoid overlap
    fig.savefig(figpath + 'linear_response.eps', bbox_inches='tight')
    plt.close()


    # plot linear response and feature importance in one graph with error bars for the paper
    fig, (ax1, ax2) = plt.subplots(1, 2, figsize=(6,2.25))

    ax1.plot(lev*0, lev, 'k:', lw=0.5)

    # temperature
    x_plot = rescale*unpack_x(sensitivity, 'T', axis=0)*per_day
    x_error = rescale*unpack_x(sensitivity_stderr, 'T', axis=0)*per_day
    ax1.plot(x_plot, lev, label='Temperature')
    #ax1.fill_betweenx(lev, x_plot-x_error, x_plot+x_error, alpha=0.5) 

    # humidity
    x_plot = rescale*unpack_x(sensitivity, 'q', axis=0)*per_day
    x_error = rescale*unpack_x(sensitivity_stderr, 'q', axis=0)*per_day
    if use_rh:
     ax1.plot(x_plot, lev, label='Relative humidity')
    else:
     ax1.plot(x_plot, lev, label='Specific humidity')
    #ax1.fill_betweenx(lev, x_plot-x_error, x_plot+x_error, alpha=0.5) 

    ax1.set_ylim([np.amax(lev), np.amin(lev)])
    ax1.set_ylabel('Vertical level ($\sigma$)')
    ax1.set_xlabel('Sensitivity (mm $\mathregular{day^{-1}}$ per perturbation)')
    ax1.legend(loc="upper right")
    ax1.legend(frameon=False)
    ax1.spines['right'].set_color('none')
    ax1.spines['top'].set_color('none')

    # weighted averaged of decrease in variance
    importance = est_eval.feature_importances_

    # simple estimate of standard error of the mean of feature importance across trees (assumes trees are independent which is not correct)
    estimators = est_eval.estimators_
    importance_stderr = np.std([tree.feature_importances_ for tree in estimators], axis=0)/np.sqrt(len(estimators))

    # account for variations in interlevel spacing
    rescale_importance = 1.0/dlev

    ax2.plot(lev*0, lev, 'k:', lw=0.5)

    # temperature
    x_plot = rescale_importance*unpack_x(importance, 'T', axis=0)
    x_error = rescale_importance*unpack_x(importance_stderr, 'T', axis=0)
    ax2.plot(x_plot, lev)
    #ax2.fill_betweenx(lev, x_plot-x_error, x_plot+x_error, alpha=0.5) 

    # humidity
    x_plot = rescale_importance*unpack_x(importance, 'q', axis=0)
    x_error = rescale_importance*unpack_x(importance_stderr, 'q', axis=0)
    ax2.plot(x_plot, lev)
    #ax2.fill_betweenx(lev, x_plot-x_error, x_plot+x_error, alpha=0.5) 

    ax2.set_ylim([np.amax(lev), np.amin(lev)])
    ax2.set_xlabel('Importance (non-dimensional)')
    #ax2.legend(loc="upper right")
    #ax2.legend(frameon=False)
    ax2.spines['right'].set_color('none')
    ax2.spines['top'].set_color('none')

    ax1.set_title('(a) Linear response', fontsize=8, loc="left")
    ax2.set_title('(b) Feature importance', fontsize=8, loc="left")

    plt.tight_layout() # avoid overlap
    fig.savefig(figpath + 'linear_response_feature_importance.eps', bbox_inches='tight')
    #fig.savefig(figpath + 'linear_response_feature_importance.png', bbox_inches='tight', dpi=600)
    plt.close()


def make_contour_plots(figpath, x_ppi, y_ppi, x_pp, y_pp, est_eval, lat, lev,
                       datafile, n_trn_exs, rain_only, no_cos, no_ps, use_rh):
    # Load data at each level
    Tmean, qmean, Tvar, qvar, Tbias, qbias, rmseT, rmseq, rT, rq, Rsq_T, Rsq_q = \
          ml_load.stats_by_latlev(x_ppi, y_ppi, x_pp, y_pp, est_eval, lat,
                                 lev, datafile, n_trn_exs, rain_only, 
                                 no_cos, no_ps, use_rh)
    # Make figs
    # True means
    f, ax1, ax2 = plot_contour(Tmean*per_day, qmean*per_day*kg_to_g, lat, lev, avg_hem=False)
    ax1.set_title(r'$\Delta$ Temp True Mean [K/day]')
    ax2.set_title(r'$\Delta$ Humid True Mean [g/kg/day]')
    plt.tight_layout() # avoid overlap
    f.savefig(figpath + 'latlev_truemean.eps', bbox_inches='tight')
    plt.close()
    # Bias from true mean
    f, ax1, ax2 = plot_contour(Tbias*per_day, qbias*per_day*kg_to_g, lat, lev, avg_hem=False)
    ax1.set_title(r'$\Delta$ Temp Mean Bias [K/day]')
    ax2.set_title(r'$\Delta$ Humid Mean Bias [g/kg/day]')
    plt.tight_layout() # avoid overlap
    f.savefig(figpath + 'latlev_bias.eps', bbox_inches='tight')
    plt.close()
    # Root mean squared error
    f, ax1, ax2 = plot_contour(rmseT*per_day, rmseq*per_day*kg_to_g, lat, lev, avg_hem=False)
    ax1.set_title(r'$\Delta$ Temp RMSE [K/day]')
    ax2.set_title(r'$\Delta$ Humid RMSE [g/kg/day]')
    plt.tight_layout() # avoid overlap
    f.savefig(figpath + 'latlev_rmse.eps', bbox_inches='tight')
    plt.close()
    # Pearson r Correlation Coefficient
    f, ax1, ax2 = plot_contour_cc(rT, rq, lat, lev, avg_hem=False)
    ax1.set_title('(a) Temperature tendency', fontsize=8)
    ax2.set_title('(b) Specific-humidity tendency', fontsize=8)
 #   plt.tight_layout() # avoid overlap
    f.savefig(figpath + 'latlev_corrcoeff.eps', bbox_inches='tight')
    plt.close()
    # Coefficient of determination
    # mask where variance is very low
    mask_too_low = 0.01
    too_low = np.nonzero(Tvar<mask_too_low*np.mean(Tvar))
    Rsq_T[too_low]=np.nan
    too_low = np.nonzero(qvar<mask_too_low*np.mean(qvar))
    Rsq_q[too_low]=np.nan
    f, ax1, ax2 = plot_contour_Rsq(Rsq_T, Rsq_q, lat, lev, avg_hem=False)
    ax1.set_title('(a) Temperature tendency', fontsize=8)
    ax2.set_title('(b) Specific-humidity tendency', fontsize=8)
    f.savefig(figpath + 'latlev_Rsq.eps', bbox_inches='tight')
    plt.close()


def plot_contour(T, q, lat, lev, avg_hem=False):
    if avg_hem:
        T, _ = ml_load.avg_hem(T, lat, 1)
        q, lat = ml_load.avg_hem(q, lat, 1)
    f, (ax1, ax2) = plt.subplots(2, sharex=True)
    cax1 = ax1.contourf(lat, lev, T)
    ax1.set_ylim(1, 0)
    ax1.set_ylabel(r'$\sigma$')
    ax1.xaxis.set_ticks([-60,-30,0,30,60])
    f.colorbar(cax1, ax=ax1)
    cax2 = ax2.contourf(lat, lev, q)
    ax2.set_ylim(1, 0)
    ax2.set_ylabel(r'$\sigma$')
    f.colorbar(cax2, ax=ax2)
    ax2.set_xlabel('Latitude (degrees)')
    ax2.xaxis.set_ticks([-60,-30,0,30,60])
    return f, ax1, ax2

def plot_contour_cc(T, q, lat, lev, avg_hem=False):
    if avg_hem:
        T, _ = ml_load.avg_hem(T, lat, 1)
        q, lat = ml_load.avg_hem(q, lat, 1)
    f, (ax1, ax2) = plt.subplots(1, 2, figsize=(6,2.25))
    levels = np.linspace(0.2,1,9)

    cax1 = ax1.contourf(lat, lev, T, levels, extend='min')
    ax1.set_ylim(1, 0)
    #cbar = f.colorbar(cax1, ax=ax1, ticks = np.arange(0.2,1.1,0.1))
    #cbar.ax.set_yticklabels(['0.2','','0.4','','0.6','','0.8','','1.0'])  
    ax1.set_ylabel(r'Vertical level ($\sigma$)')
    ax1.set_xlabel('Latitude (degrees)')
    ax1.xaxis.set_ticks([-60,-30,0,30,60])

    cax2 = ax2.contourf(lat, lev, q, levels, extend='min')
    ax2.set_ylim(1, 0)
    #cbar = f.colorbar(cax2, ax=ax2, ticks = np.arange(0.2,1.1,0.1))
    #cbar.ax.set_yticklabels(['0.2','','0.4','','0.6','','0.8','','1.0'])  
    ax2.set_xlabel('Latitude (degrees)')
    ax2.xaxis.set_ticks([-60,-30,0,30,60])

    f.subplots_adjust(right=0.9)
    cbar_ax = f.add_axes([0.93, 0.15, 0.02, 0.7])
    f.colorbar(cax2, cax=cbar_ax, ticks=np.arange(0.2,1.1,0.1))

    return f, ax1, ax2

def plot_contour_Rsq(T, q, lat, lev, avg_hem=False):
    if avg_hem:
        T, _ = ml_load.avg_hem(T, lat, 1)
        q, lat = ml_load.avg_hem(q, lat, 1)
    f, (ax1, ax2) = plt.subplots(1, 2, figsize=(6,2))
    levels = np.linspace(0.2,1,9)

    cax1 = ax1.contourf(lat, lev, T, levels, extend='min')
    ax1.set_ylim(1, min(lev))
    #cbar = f.colorbar(cax1, ax=ax1, ticks = np.arange(0.2,1.1,0.1))
    #cbar.ax.set_yticklabels(['0.2','','0.4','','0.6','','0.8','','1.0'])  
    ax1.set_ylabel(r'Vertical level ($\sigma$)')
    ax1.set_xlabel('Latitude (degrees)')
    ax1.xaxis.set_ticks([-60,-30,0,30,60])

    cax2 = ax2.contourf(lat, lev, q, levels, extend='min')
    ax2.set_ylim(1, min(lev))
    #cbar = f.colorbar(cax2, ax=ax2, ticks = np.arange(0.2,1.1,0.1))
    #cbar.ax.set_yticklabels(['0.2','','0.4','','0.6','','0.8','','1.0'])  
    ax2.set_xlabel('Latitude (degrees)')
    ax2.xaxis.set_ticks([-60,-30,0,30,60])

    f.subplots_adjust(right=0.9)
    cbar_ax = f.add_axes([0.93, 0.15, 0.02, 0.7])
    f.colorbar(cax2, cax=cbar_ax, ticks=np.arange(0.2,1.1,0.1))

    return f, ax1, ax2


# Plot means and standard deviations
def plot_means_stds(y3_true, y3_pred, lev, figpath):
    fig = plt.figure(figsize=(6.0,4.5))
    do_mean_or_std_y('mean', 'T', y3_true*per_day, y3_pred*per_day, lev, 1)
    do_mean_or_std_y('mean', 'q', y3_true*per_day*kg_to_g, y3_pred*per_day*kg_to_g, lev, 2)
    do_mean_or_std_y('std', 'T', y3_true*per_day, y3_pred*per_day, lev, 3)
    do_mean_or_std_y('std', 'q', y3_true*per_day*kg_to_g, y3_pred*per_day*kg_to_g, lev, 4)
    plt.tight_layout() # avoid overlap
    fig.savefig(figpath + 'means_stds.eps', bbox_inches='tight')
    plt.close()


# Plot correlation coefficient, explained variance, and rmse
def plot_error_stats(y3_true, y3_pred, lev, figpath):
    fig = plt.figure(figsize=(6.0,4.5))
    plt.subplot(2, 2, 1)
    plot_pearsonr(y3_true, y3_pred, 'T', lev, label='T')
    plot_pearsonr(y3_true, y3_pred, 'q', lev, label='q')
    plt.legend(loc="lower left")
    plt.legend(frameon=False)
    plt.subplot(2, 2, 2)
    plot_expl_var(y3_true, y3_pred, 'T', lev)
    plot_expl_var(y3_true, y3_pred, 'q', lev)
    plt.subplot(2, 2, 3)
    plot_rmse(y3_true*per_day, y3_pred*per_day, 'T', lev)
    plt.subplot(2, 2, 4)
    plot_rmse(y3_true*per_day*kg_to_g, y3_pred*per_day*kg_to_g, 'q', lev)
    plt.tight_layout() # avoid overlap
    fig.savefig(figpath + 'error_stats.eps', bbox_inches='tight')
    plt.close()


# Plot a time series of precipitaiton
def plot_precip(x, y_true, y_pred, dlev, figpath):
    fig = plt.figure(figsize=(3.0,2.25))
    y_true = atmos_physics.calc_precip(unpack_y(y_true, 'q'), dlev, unpack_x(x, 'ps'))
    y_pred = atmos_physics.calc_precip(unpack_y(y_pred, 'q'), dlev, unpack_x(x, 'ps'))
    ind = y_true.argsort()
    plt.plot(y_true[ind]*per_day, label='actual')
    plt.plot(y_pred[ind]*per_day, alpha=0.6, label='predict')
    plt.legend(loc="upper left")
    plt.legend(frameon=False)
    plt.title('Precipitation Rate (mm/d)')
    plt.xlabel('Sorted by actual rate')
    plt.tight_layout() # avoid overlap
    fig.savefig(figpath + 'P_rate.eps', bbox_inches='tight')
    plt.close()


# Plot a scatter plot of true vs predicted 
def plot_scatter(x, ytrue, ypred, lev, dlev, conv_cond, figpath):

    # Plot scatter of precipitation
    P_true = atmos_physics.calc_precip(unpack_y(ytrue, 'q'), dlev, unpack_x(x, 'ps'))
    P_pred = atmos_physics.calc_precip(unpack_y(ypred, 'q'), dlev, unpack_x(x, 'ps'))

    f = plt.figure(figsize=(3.0,2))
    ax = plt.gca()
    _plot_scatter(ax, P_true*per_day, P_pred*per_day)
    if conv_cond:
     ax.set_xlabel('Large-scale+convection schemes (mm $\mathregular{day^{-1}}$)')
    else:
     ax.set_xlabel('RAS convection scheme (mm $\mathregular{day^{-1}}$)')
    ax.set_ylabel('Random forest (mm $\mathregular{day^{-1}}$)')
    ax.spines['right'].set_color('none')
    ax.spines['top'].set_color('none')
    plt.tight_layout() # avoid overlap
    f.savefig(figpath + 'P_scatter.pdf', bbox_inches='tight')
    plt.close()

    Pneg = sum(P_pred < 0.0)
    Pnegpct = 100.*Pneg/len(P_pred)
    Pcc, _ = scipy.stats.pearsonr(P_true, P_pred)
    PRsq = metrics.r2_score(P_true, P_pred)
    Pbias = np.mean(P_pred-P_true)*per_day
    text_file = open(figpath+'precip.txt', 'w')
    text_file.write('Pred. P<0 %f percent of time \n' % Pnegpct)
    text_file.write('Correlation coefficient for P: %f \n' % Pcc)
    text_file.write('Coefficient of determination for P: %f \n' % PRsq)
    text_file.write('Mean bias for P: %e \n' % Pbias)
    text_file.close()

    # Plot scatters at each level
    # First create new folder
    if not os.path.exists(figpath + '/scatters/'):
        os.makedirs(figpath + '/scatters/')
    for i in range(np.size(lev)):
        f, ax = plt.subplots(1, 2)
        Ttrue = unpack_y(ytrue, 'T')[:, i]
        Tpred = unpack_y(ypred, 'T')[:, i]
        qtrue = unpack_y(ytrue, 'q')[:, i]
        qpred = unpack_y(ypred, 'q')[:, i]
        lev_str = r'$\sigma$ = {:.2f}'.format(lev[i])
        _plot_scatter(ax[0], Ttrue*per_day, Tpred*per_day, titstr='T [K/day] at '+lev_str)
        _plot_scatter(ax[1], qtrue*per_day*kg_to_g, qpred*per_day*kg_to_g, titstr='q [g/kg/day] at '+lev_str)
        Teq0 = sum(Ttrue == 0.0) / len(Ttrue) * 100.
        qeq0 = sum(qtrue == 0.0) / len(qtrue) * 100.
        ax[0].text(0.01, 0.95, 'True T=0 {:.1f}% of time'.format(Teq0),
                   transform=ax[0].transAxes)
        ax[1].text(0.01, 0.95, 'True q=0 {:.1f}% of time'.format(qeq0),
                   transform=ax[1].transAxes)
        plt.tight_layout() # avoid overlap
        f.savefig(figpath + '/scatters/Tq_scatter_sigma{:.2f}.png'
                  .format(lev[i]), bbox_inches='tight', dpi=600)
        plt.close()


def _plot_scatter(ax, true, pred, titstr=None):
    ax.scatter(true, pred, s=5, c='b', alpha=0.2)
    # Calculate mins and maxs and set axis bounds appropriately
    xmin = np.min(true)
    xmax = np.max(true)
    ymin = np.min(pred)
    ymax = np.max(pred)
    xymin = np.min([xmin, ymin])
    xymax = np.max([xmax, ymax])
    # Plot 1-1 line
    ax.plot([xymin, xymax], [xymin, xymax], color='k', ls='--')
    ax.set_xlabel('True')
    ax.set_ylabel('Predicted')
    if titstr is not None:
        ax.set_title(titstr)


# Plot the enthalpy conservation
def plot_enthalpy(x, y_true, y_pred, dlev, figpath):
    fig = plt.figure(figsize=(3.0,4.5))
    plt.subplot(2, 1, 1)
    _plot_enthalpy(x, y_true, dlev, label='true')
    plt.legend(loc="upper left")
    plt.legend(frameon=False)
    plt.subplot(2, 1, 2)
    _plot_enthalpy(x, y_pred, dlev, label='predict')
    plt.legend(loc="upper left")
    plt.legend(frameon=False)
    plt.tight_layout() # avoid overlap
    fig.savefig(figpath + 'enthalpy_conservation.png', bbox_inches='tight', dpi=600)
    plt.close()

# ----  PLOTTING SCRIPTS  ---- #


def do_mean_or_std_y(method, vari, true, pred, lev, ind):
    methods = {'mean': np.mean, 'std': np.std}
    methods_ti = {'mean': 'Mean', 'std': 'Standard Deviation'}
    plt.subplot(2, 2, ind)
    m = lambda y: methods[method](unpack_y(y, vari), axis=0).T
    plt.plot(m(true), lev, label='true')
    plt.plot(m(pred), lev, label='pred')
    plt.ylim(np.amax(lev), np.amin(lev))
    plt.ylabel('$\sigma$')
    out_str_dict = {'T': 'K/day', 'q': 'g/kg/day'}
    if ind > 2:
        plt.xlabel(out_str_dict[vari])
    plt.title(r'$\Delta$ ' + vari + " " + methods_ti[method])
    plt.legend()
    plt.legend(frameon=False)


def plot_pearsonr(y_true, y_pred, vari, lev, label=None):
    r = np.empty(y_true.shape[1])
    prob = np.empty(y_true.shape[1])
    for i in range(y_true.shape[1]):
        r[i], prob[i] = scipy.stats.pearsonr(y_true[:, i], y_pred[:, i])
    plt.plot(unpack_y(r, vari, axis=0), lev, label=label)
    plt.ylim([np.amax(lev), np.amin(lev)])
    plt.xlim(0, 1)
    plt.ylabel('$\sigma$')
    plt.title('Correlation Coefficient')


def plot_rmse(y_true, y_pred, vari, lev, label=None):
    rmse = np.sqrt(metrics.mean_squared_error(y_true, y_pred,
                                              multioutput='raw_values'))
    plt.plot(unpack_y(rmse, vari, axis=0), lev, label=label)
    plt.ylim([np.amax(lev), np.amin(lev)])
    plt.xlim(xmin=0)
    plt.ylabel('$\sigma$')
    out_str_dict = {'T': 'K$^2$/day$^2$', 'q': 'g$^2$/kg$^2$/day$^2$'}
    plt.xlabel(out_str_dict[vari])
    plt.title('Root Mean Squared Error')


def plot_expl_var(y_true, y_pred, vari, lev, label=None):
    expl_var = metrics.explained_variance_score(y_true, y_pred,
                                                multioutput='raw_values')
    plt.plot(unpack_y(expl_var, vari, axis=0), lev, label=label)
    plt.ylim([np.amax(lev), np.amin(lev)])
    plt.xlim(0,1)
    plt.ylabel('$\sigma$')
    plt.title('Explained Variance Regression Score')


def _plot_enthalpy(x, y, dlev, label=None):
    k = atmos_physics.calc_enthalpy(unpack_y(y, 'T'), unpack_y(y, 'q'), dlev)
    n, bins, patches = plt.hist(k*per_day, 50, alpha=0.5, label=label)
    energy_rate = atmos_physics.calc_energy_conservation(unpack_y(y, 'T'), unpack_y(y, 'q'), dlev, unpack_x(x, 'ps'))
    plt.title('RMS heating rate {:1.2e} K/day ({:1.2e} W/m^2)'.format(np.sqrt(np.mean(k**2))*per_day, np.sqrt(np.mean(energy_rate**2))))
    plt.xlabel('K/day over column')


def check_scaling_distribution(x, x_scl, y, y_scl, lat, lev,
                               figpath, use_rh=False):
    # For input variables
    fig, ax = plt.subplots(2, 2)
    _plot_distribution(unpack_x(x, 'T'), lat, lev, fig, ax[0, 0], './figs/', 'T (unscaled) [K]', '')

    _plot_distribution(unpack_x(x_scl, 'T'), lat, lev, fig, ax[0, 1], './figs/', 'T (scaled) []', '')

    if use_rh:
     _plot_distribution(unpack_x(x, 'q'), lat, lev, fig, ax[1, 0], './figs/', 'rh (unscaled)', '')
    else:
     _plot_distribution(unpack_x(x, 'q')*kg_to_g, lat, lev, fig, ax[1, 0], './figs/', 'q (unscaled) [g/kg]', '')

    _plot_distribution(unpack_x(x_scl, 'q'), lat, lev, fig, ax[1, 1], './figs/', 'q (scaled) []', '')

    plt.tight_layout() # avoid overlap
    fig.savefig(figpath + 'input_scaling_check.png', bbox_inches='tight',
                dpi=450)
    plt.close()
    # For output variables
    fig, ax = plt.subplots(2, 2)
    _plot_distribution(unpack_y(y, 'T')*per_day, lat, lev, fig, ax[0, 0], './figs/', 'T tend (unscaled) [K/day]', '')

    _plot_distribution(unpack_y(y_scl, 'T'), lat, lev, fig, ax[0, 1], './figs/', 'T tend (scaled) []', '')

    _plot_distribution(unpack_y(y, 'q')*per_day*kg_to_g, lat, lev, fig, ax[1, 0], './figs/', 'q tend (unscaled) [g/kg/day]', '')

    _plot_distribution(unpack_y(y_scl, 'q'), lat, lev, fig, ax[1, 1], './figs/', 'q tend(scaled) []', '')

    plt.tight_layout() # avoid overlap
    fig.savefig(figpath + 'output_scaling_check.png', bbox_inches='tight',
                dpi=450)
    plt.close()


def check_output_distribution(yt, yt_scl, yp, yp_scl, lat, lev,
                              figpath):
    # For unscaled variables
    fig, ax = plt.subplots(2, 2)
    x1, x2, bins = _plot_distribution(unpack_y(yt, 'T')*per_day, lat, lev, fig,
                                      ax[0, 0], './figs/',
                                      r'$\Delta$T true [K/day]', '')
    _plot_distribution(unpack_y(yp, 'T')*per_day, lat, lev, fig,
                       ax[0, 1], './figs/', r'$\Delta$T pred [K/day]', '', x1,
                       x2, bins)
    x1, x2, bins = _plot_distribution(unpack_y(yt, 'q')*per_day*kg_to_g, lat, lev, fig,
                                      ax[1, 0], './figs/',
                                      r'$\Delta$q true [g/kg/day]', '')
    _plot_distribution(unpack_y(yp, 'q')*per_day*kg_to_g, lat, lev, fig, ax[1, 1],
                       './figs/', r'$\Delta$q pred [g/kg/day]', '', x1, x2,
                       bins)
    plt.tight_layout() # avoid overlap
    fig.savefig(figpath + 'output_compare_true_pred_unscaled.png',
                bbox_inches='tight', dpi=450)
    plt.close()
    # For scaled variables
    fig, ax = plt.subplots(2, 2)
    x1, x2, bins = _plot_distribution(unpack_y(yt_scl, 'T'), lat, lev, fig,
                                      ax[0, 0], './figs/',
                                      r'$\Delta$T true (scld) []', '')
    _plot_distribution(unpack_y(yp_scl, 'T'), lat, lev, fig, ax[0, 1], './figs/',
                       r'$\Delta$T pred (scld) []', '', x1, x2, bins)
    x1, x2, bins = _plot_distribution(unpack_y(yt_scl, 'q'), lat, lev, fig,
                                      ax[1, 0], './figs/',
                                      r'$\Delta$q true (scld) []', '')
    _plot_distribution(unpack_y(yp_scl, 'q'), lat, lev, fig, ax[1, 1], './figs/',
                       r'$\Delta$q pred (scld) []', '', x1, x2, bins)
    plt.tight_layout() # avoid overlap
    fig.savefig(figpath + 'output_compare_true_pred_scaled.png',
                bbox_inches='tight', dpi=450)
    plt.close()


def _plot_distribution(z, lat, lev, fig, ax, figpath, titlestr, xstr, xl=None,
                       xu=None, bins=None):
    """Plots a stack of histograms of log10(data) at all levels"""
    # Initialize the bins and the frequency
    num_bins = 100
    if bins is None:
        bins = np.linspace(np.percentile(z, .02), np.percentile(z, 99.98),
                           num_bins+1)
    n = np.zeros((num_bins, lev.size))
    # Calculate distribution at each level
    for i in range(lev.size):
        n[:, i], _ = np.histogram(z[:, i], bins=bins)
    # Take a logarithm and deal with case where we take log of 0
    n = np.log10(n)
    n_small = np.amin(n[np.isfinite(n)])
    n[np.isinf(n)] = n_small
    # Plot histogram
    ca = ax.contourf(bins[:-1], lev, n.T)
    ax.set_ylim(1, 0)
    if xl is not None:
        ax.set_xlim(xl, xu)
    plt.colorbar(ca, ax=ax)
    ax.set_xlabel(xstr)
    ax.set_ylabel(r'$\sigma$')
    ax.set_title(titlestr)
    xl, xr = ax.set_xlim()
    return xl, xr, bins


def plot_sample_profiles(num_prof, x, ytrue, ypred, lev, figpath, samp=None):
    # Make directory if one does not exist
    if not os.path.exists(figpath + '/samples/'):
        os.makedirs(figpath + '/samples/')
    # Plot some number of sample profiles
    for i in range(num_prof):
        if samp is None:
            samp = np.random.randint(0, x.shape[0])
        sample_filename = figpath + '/samples/' + str(samp) + '.eps'
        plot_sample_profile(x[samp, :], ytrue[samp, :], ypred[samp, :],
                            lev, filename=sample_filename)
        # Need to reset sample number for next iteration
        samp = None


def plot_sample_profile(x, y_true, y_pred, lev, filename=None, pflag=False):
        """Plots the vertical profiles of input T & q and predicted and true
        output tendencies"""
        f, (ax1, ax3) = plt.subplots(1, 2, figsize=(7.5,5))
        T = unpack_x(x, 'T', axis=0)
        q = unpack_x(x, 'q', axis=0)
        ps = unpack_x(x, 'ps', axis=0)
        theta = atmos_physics.calc_theta(T, lev, ps)
        # Plot input temperature profile
        if pflag:
            lev = lev*1000
        ax1.plot(theta, lev, label=r'$\theta$')
        if pflag:
         ax1.set_ylim(1000, 250)
        else:
         ax1.set_ylim(1, 0.1)
        ax1.set_xlim(270, 370)
        ax1.set_title(r'Input Profiles')
        ax1.set_xlabel(r'$\theta$ [K]')
        ax1.grid(True)
        ax1.legend(loc='upper left')
        ax1.legend(frameon=False)
        cp = atmos_physics.cp
        L = atmos_physics.L 
        kJ_scale = 0.001
        k_per_day = 3600*24
        ax3.plot(cp*ml_load.unpack_y(y_true, 'T', axis=0)*per_day*kJ_scale, lev, color='red',
                 ls='-', label=r'$\Delta$T true')
        ax3.plot(cp*ml_load.unpack_y(y_pred, 'T', axis=0)*per_day*kJ_scale, lev, color='red',
                 ls='--', label=r'$\Delta$T pred')
        ax3.plot(L*ml_load.unpack_y(y_true, 'q', axis=0)*per_day*kJ_scale, lev, color='blue',
                 ls='-', label=r'$\Delta$q true')
        ax3.plot(L*ml_load.unpack_y(y_pred, 'q', axis=0)*per_day*kJ_scale, lev, color='blue',
                 ls='--', label=r'$\Delta$q pred')
        if pflag:
         ax3.set_ylim(1000, 250)
        else:
         ax3.set_ylim(1, 0.1)
        ax3.set_xlabel('Cp*T or L*q [kJ/day/kg]')
        ax1.set_ylabel('Pressure [hPa]')
        ax3.set_title('Output Tendencies')
        ax3.legend(loc="upper left")
        ax3.legend(frameon=False)
        ax3.grid(True)
        f.tight_layout()
        # Save file if requested
        if filename is not None:
            f.savefig(filename, bbox_inches='tight')
            plt.close()


def plot_model_error_over_time(errors, est_str, fig_dir):
    x = np.arange(errors.shape[0])
    ytix = [.5e-3, 1e-3, 2e-3, 5e-3, 10e-3, 20e-3, 50e-3, 100e-3]
    # Plot error rate vs. iteration number
    fig = plt.figure(figsize=(3.0,2.25))
    # Plot training errors from cost function
    plt.semilogy(x, np.squeeze(errors[:, 0]), alpha=0.5, color='blue',
                 label='Training (cost function)')
    plt.yticks(ytix, ytix)
    plt.ylim((np.nanmin(errors), np.nanmax(errors)))
    # Plot training errors that are not associated with cost function
    plt.semilogy(x, np.squeeze(errors[:, 4]), alpha=0.5, color='red',
                 label='Training')
    # Plot cross-validation errors
    plt.semilogy(x, np.squeeze(errors[:, 2]), alpha=0.5, color='green',
                 label='Cross-Val')
    plt.legend()
    plt.legend(frameon=False)
    plt.title('Error for ' + est_str)
    plt.xlabel('Iteration Number')
    fig.savefig(fig_dir + 'error_history.png', bbox_inches='tight', dpi=450)
    plt.close()

def calc_mse_simple(y_pred, y_true, relflag=False, min_lev=None, lev=None):
    """Calculates the mean squared error for scaled outputs and is taken over
       all training examples and levels of T and q
       Args:
        y_pred (float: Nex x n_lev*2): Scaled predicted value of output
        y_true (float Nex x n_lev*2): Scaled true value of output
        relflag (bool): Divide by standard deviation for each var at each lev
                        to somewhat account for difference in strength between
                        convection-only and conv+cond predictions
        min_lev (float): Don't consider data above this level when calculating
                        mse
        lev (float: n_lev): Used for calculating min_lev
       Returns:
        mse (float, scalar): Mean-Squared error """
    # If requested cut off certain levels
    if min_lev is not None:
        if lev is None:
            raise ValueError('Also need to input levels!')
        indlev = np.greater_equal(lev, min_lev)
        # For predicted data
        Tp = unpack_y(y_pred, 'T')[:, indlev]
        qp = unpack_y(y_pred, 'q')[:, indlev]
        y_pred = pack_y(Tp, qp)
        # For true data
        Tt = unpack_y(y_true, 'T')[:, indlev]
        qt = unpack_y(y_true, 'q')[:, indlev]
        y_true = pack_y(Tt, qt)
    # Calculate squared difference
    mse = np.square(y_pred - y_true)
    # If looking at relative mean squared error divide by the mean over all
    # examples for each variable and at each level
    if relflag:
        mse = mse/np.abs(np.std(y_true, axis=1))[:, None]
    # Now take mean value over flattened array for all values
    mse = np.mean(mse[np.isfinite(mse)])
    return mse


