import numpy as np
import sknn_jgd.mlp
import time
from sklearn.ensemble import RandomForestRegressor
#from sklearn.tree import DecisionTreeRegressor
from src.ml_io import write_netcdf_rf
from src.ml_io import write_netcdf_nn
import src.ml_load as ml_load
import pickle
import src.ml_plot as ml_plot
import os

# ---  build random forest or neural net  --- #
def train_wrapper(x_ppi, y_ppi, training_expt,
                  do_nn=False, n_iter=None, do_train=True, 
                  no_cos=False, no_ps=False, use_rh=False,
                  min_lev=0.0,  
                  rain_only=False, 
                  n_trn_exs=None, conv_cond=False, 
                  plot_training_results=False, 
                  n_trees = 100, min_samples_leaf = 10, 
                  n_layers=2, n_hid_neur=10, n_stable=None, weight_decay=0.0):
    """Loads training data and trains and stores estimator

    Args:
        x_ppi (dict): The type of preprocessing to do to the features (inputs)
        y_ppi (dict): The type of preprocessing to do to the targets (outputs)
        n_layers (int): Number of layers in the NN
        n_hid_neur (int): Number of hidden neurons in each layer
        n_iter (int): Number of iterations
        n_stable (int): Number of iterations after stability reached
        min_lev (float): Don't train on data above this sigma level (assume 0)
        weight_decay (float): Regularization strength. 0 is no regularization
        rain_only (bool): Only train on precipitating examples
        n_trn_exs (int): Number of training examples to learn on
        conv_cond (bool): If true, learn to do convection + condensation
        do_nn (bool): Use an ANN instead of a random forest 
        no_cos (bool): If true, don't weight by cosine(latitude)
        min_samples_leaf (int): minimum samples per leaf
        plot_training_results (bool): Whether to also plot the model on training data
        no_ps (bool): don't include surface pressure as a feature by setting ps_scl to a constant value of zero (not quite same as excluding it as feature as effectively changes max_features for RF)
        use_rh (bool): use relative humidity instead of specific humidity as feature
        do_train (bool): whether to train (just plot the results if false)
    Returns:
        str: String id of trained NN
    """

    # Load data (note LoadData seeds the random number generator)
    datadir, trainfile, testfile, pp_str = ml_load.GetDataPath(conv_cond, training_expt)
    x, y, cv, Pout, lat, lev, dlev = ml_load.LoadData(trainfile, min_lev, rain_only=rain_only, n_trn_exs=n_trn_exs, no_cos=no_cos, use_rh=use_rh)


    # Scale data
    x_pp, x_scl, y_pp, y_scl, pp_str = PreprocessData(x_ppi, x, y_ppi, y, pp_str, n_trn_exs, lev)

    # Either build a random forest or build a neural netowrk
    if do_nn:
        regularize = CatchRegularization(weight_decay)
        est, est_str = BuildNN(min_lev, n_layers, 'Rectifier', n_hid_neur,
                                'momentum', pp_str, batch_size=100,
                                n_stable=n_stable, n_iter=n_iter,
                                learning_momentum=0.9, learning_rate=0.01,
                                regularize=regularize,
                                weight_decay=weight_decay,
                                valid_size=0.2)
    else:
        est, est_str = BuildRandomForest(min_lev, n_trees, min_samples_leaf, 
                                         pp_str)

    est_str = UpdateName(no_cos, no_ps, use_rh, rain_only, est_str)

    # Print details about the ML algorithm we are using
    print(est_str + ' Using ' + str(x.shape[0]) + ' training examples with ' +
          str(x.shape[1]) + ' input features and ' + str(y.shape[1]) +
          ' output targets')
    if no_ps:
     print('But ps_scl set to zero')

    # Train the estimator
    if do_train:
      est, est_errors = train_est(est, est_str, x_scl, y_scl, do_nn, no_ps)
      # Save the estimator to access it later
      save_est(est, est_str, est_errors, x_ppi, y_ppi, x_pp, y_pp, lat, lev, dlev)
    # Write a netcdf file for the gcm
    if do_nn:
     write_netcdf_nn(est_str, trainfile, rain_only, no_cos, use_rh)
    else:
     write_netcdf_rf(est_str, trainfile, rain_only, no_cos, use_rh)

    # Plot figures with testing data using all of it
    figpath = './figs/' + est_str + '/'
    ml_plot.PlotAllFigs(est_str, testfile, do_nn, figpath, rain_only=rain_only, 
                       no_cos=no_cos, no_ps=no_ps, use_rh=use_rh, 
                       conv_cond=conv_cond)

    if plot_training_results: # note use n_trn_exs here as training data
        figpath = figpath + 'training_data/'
        ml_plot.PlotAllFigs(est_str, trainfile, do_nn, figpath,
                            rain_only=rain_only, n_trn_exs=n_trn_exs,
                            no_cos=no_cos, no_ps=no_ps, use_rh=use_rh, 
                            conv_cond=conv_cond)
    return est_str



def PreprocessData(x_ppi, x, y_ppi, y, pp_str, n_trn_exs, lev):
    """Transform data according to input preprocessor requirements and make
    make preprocessor string for saving"""
    x_pp = ml_load.init_pp(x_ppi, x)
    x = ml_load.transform_data(x_ppi, x_pp, x, lev)
    y_pp = ml_load.init_pp(y_ppi, y)
    y = ml_load.transform_data(y_ppi, y_pp, y, lev)
    # Make preprocessor string for saving
    pp_str = pp_str + 'X-' + x_ppi['name'] + '_'
    pp_str = pp_str + 'Y-' + y_ppi['name'] + '_'
    # Add number of training examples to string
    pp_str = pp_str + 'Ntrnex' + str(n_trn_exs) + '_'
    return x_pp, x, y_pp, y, pp_str

def CatchRegularization(weight_decay):
    """scikit-neuralnetwork seems to have a bug if regularization is set to zero"""
    if weight_decay > 0.0:
        regularize = 'L2'
    else:
        regularize = None
    return regularize

def UpdateName(no_cos, no_ps, use_rh, rain_only, est_str):
    if no_cos:
        est_str = est_str + '_nocos'
    if no_ps:
        est_str = est_str + '_nops'
    if use_rh:
        est_str = est_str + '_rh'
    if rain_only:
        est_str = est_str + '_rain'

    # xtropics20 or tropics20 (also change ml_load.py)
    #est_str = est_str + '_xtropics20'
    #est_str = est_str + '_tropics20'

    return est_str

def save_est(est, est_str, est_errors, x_ppi, y_ppi, x_pp, y_pp, lat, lev, dlev):
    """Save estimator"""
    if not os.path.exists('/net/esker/archive1/pog/mldata/regressors/'):
        os.makedirs('/net/esker/archive1/pog/mldata/regressors/')
    pickle.dump([est, est_str, est_errors, x_ppi, y_ppi, x_pp, y_pp, lat, lev,
                 dlev], open('/net/esker/archive1/pog/mldata/regressors/' + est_str + '.pkl', 'wb'))

def store_stats(i, avg_train_error, best_train_error, avg_valid_error,
                best_valid_error, avg_train_obj_error, best_train_obj_error,
                **_):
    if i == 1:
        global errors_stored
        errors_stored = []
    errors_stored.append((avg_train_error, best_train_error,
                          avg_valid_error, best_valid_error,
                          avg_train_obj_error, best_train_obj_error))


def BuildNN(min_lev, n_layers, actv_fnc, hid_neur, learning_rule, pp_str,
             batch_size=100, n_iter=None, n_stable=None,
             learning_rate=0.01, learning_momentum=0.9,
             regularize='L2', weight_decay=0.0, valid_size=0.5,
             f_stable=.001):
    """Builds a multi-layer perceptron via the scikit neural network interface
       Note using lasagne and theano
    """
    # First build layers
    actv_fnc = n_layers*[actv_fnc]
    hid_neur = n_layers*[hid_neur]
    layers = [sknn_jgd.mlp.Layer(f, units=h) for f, h in zip(actv_fnc,
                                                             hid_neur)]
    # Append a linear output layer 
    layers.append(sknn_jgd.mlp.Layer("Linear"))
    est = sknn_jgd.mlp.Regressor(layers,
                                 n_iter=n_iter,
                                 batch_size=batch_size,
                                 learning_rule=learning_rule,
                                 learning_rate=learning_rate,
                                 learning_momentum=learning_momentum,
                                 regularize=regularize,
                                 weight_decay=weight_decay,
                                 n_stable=n_stable,
                                 valid_size=valid_size,
                                 f_stable=f_stable,
                                 callback={'on_epoch_finish': store_stats})
    # Write nn string
    layerstr = '_'.join([str(h) + f[0] for h, f in zip(hid_neur, actv_fnc)])
    if learning_rule == 'momentum':
        lrn_str = str(learning_momentum)
    else:
        lrn_str = str(learning_rate)
    # Construct name
    est_str = pp_str + "r_" + layerstr + "_" + learning_rule[0:3] +\
        lrn_str
    # If using regularization, add that to the name too
    if weight_decay > 0.0:
        est_str = est_str + 'reg' + str(weight_decay)
    # Add the number of iterations too
    est_str = est_str + '_Niter' + str(n_iter)
    # Add the minimum level
    est_str = est_str + '_minlev' + str(min_lev)
    return est, est_str


def BuildRandomForest(min_lev, n_trees, min_samples_leaf, pp_str):

    est = RandomForestRegressor(n_estimators=n_trees, min_samples_leaf = min_samples_leaf, max_features = 1.0/3.0, random_state = 123)
    est_str = pp_str + 'RF_regress_NumTrees' + str(n_trees) + '_MinSamples' + str(min_samples_leaf) + '_MaxFeaturesThird' + '_minlev' + str(min_lev)

    return est, est_str


def train_est(est, est_str, x_scl, y_scl, do_nn, no_ps=False):
    """Train estimator"""

    # Initialize
    start = time.time()

    # Train the model using training data
    if no_ps: # set ps_scl to a constant for training
     T_scl = ml_load.unpack_x(x_scl, 'T')
     q_scl = ml_load.unpack_x(x_scl, 'q')
     ps_scl = ml_load.unpack_x(x_scl, 'ps')
     est.fit(ml_load.pack_x(T_scl,q_scl,np.zeros(ps_scl.shape)), y_scl)
    else:
     est.fit(x_scl, y_scl)
      
    train_score = est.score(x_scl, y_scl)
    end = time.time()
    print("Training Score: {:.4f} for Model {:s} ({:.1f} seconds)".format(
                                              train_score, est_str, end-start))
    if do_nn:
     # This is an n_iter x 4 array...see score_stats
     errors = np.asarray(errors_stored)
    else:
     errors = np.empty(0)

    # Return the fitted models and the scores
    return est, errors


